'use strict';

const Etcd = require('node-etcd');

module.exports = class EtcdConfigurator {
  constructor(ravelApp, path = 'ravelrc') {
    path = `/${path.replace('/', '')}/`; // convert path to proper format (i.e. /path/)
    ravelApp.registerParameter('config etcd host', true, '127.0.0.1:2379');
    ravelApp.registerParameter('config etcd options', true, {timeout: 5000});

    ravelApp.on('post load parameters', () => {
      const logger = ravelApp.log.getLogger(this.constructor.name);
      const host = ravelApp.get('config etcd host');
      logger.info(`Loading configuration from etcd at ${host}`);
      const etcd = new Etcd(host, ravelApp.get('config etcd options'));

      // load parameters from folder in etcd
      const result = etcd.getSync(path, {recursive: true});
      if (result.err) {
        logger.error(`Unable to load configuration from etcd at ${host} with path ${path}`, result.err);
        throw result.err;
      } else if (!result.body || !result.body.node || !result.body.node.nodes) {
        logger.error(`Expected etcd at ${host} to contain configuration values in folder ${path}`);
        throw new Error(`Expected etcd at ${host} to contain configuration values in folder ${path}`);
      } else {
        // set parameters on app (and try to JSON parse them)
        for (let val of result.body.node.nodes) {
          const key = val.key.substring(path.length);
          logger.trace(`Setting parameter ${key} to value ${val.value}`);
          try {
            ravelApp.set(key, JSON.parse(val.value));
          } catch(err2) {
            ravelApp.set(key, val.value);
          }
        }
      }
    });
  }
};
